// Copyright The Notary Project Authors.
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package io

import (
	"encoding/json"
	"os"
)

// UnmarshalRequest read the STDIN and unmarshal to struct pointed by req.
func UnmarshalRequest(req any) error {
	return json.NewDecoder(os.Stdin).Decode(req)
}

// PrintResponse marshal and print the struct data pointed by resp.
func PrintResponse(resp any) error {
	return json.NewEncoder(os.Stdout).Encode(resp)
}
