// Copyright The Notary Project Authors.
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package mock

const (
	TamperKeyID                 = "TAMPER_KEY_ID"
	TamperSignature             = "TAMPER_SIGNATURE"
	TamperSignatureAlgorithm    = "TAMPER_SIGNATURE_ALGORITHM"
	TamperCertificateChain      = "TAMPER_CERTIFICATE_CHAIN"
	TamperSignatureEnvelope     = "TAMPER_SIGNATURE_ENVELOPE"
	TamperSignatureEnvelopeType = "TAMPER_SIGNATURE_ENVELOPE_TYPE"
	TamperAnnotation            = "TAMPER_ANNOTATION"
)
